@@ -21,6 +21,7 @@
 This module provides the base implementation for pyRFXtrx
 """
 # pylint: disable=R0903, invalid-name
+# pylint: disable= too-many-lines
 
 import glob
 import socket
@@ -166,6 +167,94 @@ class RfyDevice(RFXtrxDevice):
         self.send_command(transport, 0x12)
 
 
+class FunkDevice(RFXtrxDevice):
+    """ Concrete class for a control device """
+
+    def __init__(self, pkt):
+        super().__init__(pkt)
+        if isinstance(pkt, lowlevel.Funkbus):
+            self.id_combined = pkt.id_combined
+            self.groupcode = pkt.groupcode
+            self.target = pkt.target
+            self.COMMANDS = lowlevel.Funkbus.COMMANDS
+
+    def send_command(self, transport, command, param, duration):
+        """ Send a command using the given transport """
+        pkt = lowlevel.Funkbus()
+        pkt.set_transmit(self.subtype,
+                         0,
+                         self.id_combined,
+                         self.groupcode,
+                         param if command in [0x00, 0x01, 0x04] else 0x00,
+                         command,
+                         duration)
+        transport.send(pkt.data)
+
+    def send_onoff(self, transport, turn_on):
+        """ Send on 'On' or 'Off' command using the given transport """
+        self.send_command(transport,
+                          0x01 if turn_on else 0x00,
+                          self.target,
+                          0x00)
+
+    def send_on(self, transport):
+        """ Send an 'On' command using the given transport """
+        self.send_onoff(transport, True)
+
+    def send_off(self, transport):
+        """ Send an 'Off' command using the given transport """
+        self.send_onoff(transport, False)
+
+    def send_dim(self, transport, duration):
+        """ Send a 'Dim' command using the given transport """
+        self.send_command(transport,
+                          0x00,
+                          self.target,
+                          duration + 1)
+
+    def send_bright(self, transport, duration):
+        """ Send a 'Bright' command using the given transport """
+        self.send_command(transport,
+                          0x01,
+                          self.target,
+                          duration + 1)
+
+    def send_alloff(self, transport):
+        """ Send an 'All OFF' command using the given transport """
+        self.send_command(transport,
+                          0x02,
+                          0x00,
+                          0x03)
+
+    def send_allon(self, transport):
+        """ Send a 'All ON' command using the given transport """
+        self.send_command(transport,
+                          0x03,
+                          0x00,
+                          0x03)
+
+    def send_setscene(self, transport, scene):
+        """ Send a 'Scene' command using the given transport """
+        self.send_command(transport,
+                          0x04,
+                          scene,
+                          0x01)
+
+    def send_masterdim(self, transport, duration):
+        """ Send a 'Master Dim' command using the given transport """
+        self.send_command(transport,
+                          0x05,
+                          0x00,
+                          duration + 1)
+
+    def send_masterbright(self, transport, duration):
+        """ Send a 'Bright' command using the given transport """
+        self.send_command(transport,
+                          0x06,
+                          0x00,
+                          duration + 1)
+
+
 class LightingDevice(RFXtrxDevice):
     """ Concrete class for a control device """
 
@@ -335,6 +424,8 @@ class LightingDevice(RFXtrxDevice):
                 transport.send(pkt.data)
         elif self.packettype == 0x15:  # Lighting6
             raise ValueError("Dim level unsupported for Lighting6")
+        elif self.packettype == 0x1e:  # Funkbus
+            raise ValueError("Dim level unsupported for Funkbus")
         else:
             raise ValueError("Unsupported packettype")
 
@@ -373,6 +464,8 @@ def get_device_from_pkt(pkt):
         device = ChimeDevice(pkt)
     elif isinstance(pkt, lowlevel.Security1):
         device = SecurityDevice(pkt)
+    elif isinstance(pkt, lowlevel.Funkbus):
+        device = FunkDevice(pkt)
     else:
         device = RFXtrxDevice(pkt)
     return device
@@ -510,9 +603,17 @@ class SensorEvent(RFXtrxEvent):
                 self.values['Voltage'] = pkt.voltage
                 self.values['Energy usage'] = pkt.currentwatt
                 self.values['Sensor Status'] = pkt.teleinfo_ok
+            elif pkt.type_string == 'CARTELECTRONIC_TIC':
+                self.values['Counter value'] = pkt.counter1
+                self.values['Count'] = pkt.counter2
+                self.values['Energy usage'] = pkt.currentwatt
+                self.values['Sensor Status'] = pkt.teleinfo_ok
+                self.values['Contract type'] = pkt.contract_type
         if isinstance(pkt, lowlevel.Security1):
             self.values['Sensor Status'] = pkt.security1_status_string
-        if not isinstance(pkt, (lowlevel.Energy5, lowlevel.RfxMeter, lowlevel.Undecoded)):
+        if not isinstance(pkt, (lowlevel.Energy5,
+                                lowlevel.RfxMeter,
+                                lowlevel.Undecoded)):
             self.values['Battery numeric'] = pkt.battery
         if not isinstance(pkt, lowlevel.Undecoded):
             self.values["Rssi numeric"] = pkt.rssi
@@ -555,6 +656,9 @@ class ControlEvent(RFXtrxEvent):
         if pkt.rssi is not None:
             self.values['Rssi numeric'] = pkt.rssi
 
+        if isinstance(pkt, lowlevel.Funkbus):
+            self.values['Keypress'] = pkt.value('time_string')
+
     def __str__(self):
         return "{0} device=[{1}] values={2}".format(
             type(self), self.device, sorted(self.values.items()))
@@ -602,7 +706,7 @@ class _dummySerial:
         self._data[7] = [0x0a, 0x20, 0x00, 0x00, 0x00,
                          0x00, 0x00, 0x00, 0x00, 0x00, 0x00]  # sensor2
         self._data[8] = [0x09, 0x03, 0x01, 0x1e,
-                         0x28, 0x0a, 0xb7, 0x66, 0x04, 0x74] # undecoded
+                         0x28, 0x0a, 0xb7, 0x66, 0x04, 0x74]  # undecoded
         self._close_event = threading.Event()
 
     def write(self, *args, **kwargs):
